@@ -1,17 +1,22 @@
+mod consts;
+pub mod opcode;
+
 pub use crate::{CoinData, CoinID, Transaction};
 use crate::{CoinDataHeight, Denom, Header, HexBytes};
+
+use std::{collections::HashMap, str::FromStr};
+use std::{convert::TryInto, fmt::Display};
+
 use arbitrary::Arbitrary;
 use catvec::CatVec;
 use derive_more::{From, Into};
 use ethnum::U256;
 use serde::{Deserialize, Serialize};
-use std::{collections::HashMap, str::FromStr};
-use std::{convert::TryInto, fmt::Display};
 use tap::Tap;
 use thiserror::Error;
 use tmelcrypt::HashVal;
 
-use self::{
+use crate::melvm::{
     consts::{
         HADDR_LAST_HEADER, HADDR_PARENT_ADDITIONAL_DATA, HADDR_PARENT_DENOM, HADDR_PARENT_HEIGHT,
         HADDR_PARENT_INDEX, HADDR_PARENT_TXHASH, HADDR_PARENT_VALUE, HADDR_SELF_HASH,
@@ -19,8 +24,7 @@ use self::{
     },
     opcode::{opcodes_weight, DecodeError, EncodeError, OpCode},
 };
-mod consts;
-pub mod opcode;
+
 
 #[derive(Clone, Eq, PartialEq, Debug, Arbitrary, Serialize, Deserialize, Hash)]
 /// A MelVM covenant. Essentially, given a transaction that attempts to spend it, it either allows the transaction through or doesn't.
@@ -85,6 +89,40 @@ pub struct CovenantEnv<'a> {
 }
 
 impl Covenant {
+    /// Converts to a vector of OpCodes.
+    pub fn to_ops(&self) -> Result<Vec<OpCode>, DecodeError> {
+        let mut opcodes: Vec<OpCode> = Vec::with_capacity(128);
+
+        let mut temporary_slice: &[u8] = self.0.as_slice();
+
+        while !temporary_slice.is_empty() {
+            opcodes.push(OpCode::decode(&mut temporary_slice)?);
+        }
+
+        Ok(opcodes)
+    }
+
+    /// Create a Covenant from a slice of OpCodes.
+    pub fn from_ops(ops: &[OpCode]) -> Result<Self, EncodeError> {
+        let mut output: Vec<u8> = Vec::new();
+        // go through output
+        let was_encoding_successful: Result<(), EncodeError> = ops.iter().try_for_each(|op| {
+            match op.encode() {
+                Ok(data) => {
+                    output.extend_from_slice(&data);
+
+                    Ok(())
+                },
+                Err(error) => Err(error),
+            }
+        });
+
+        match was_encoding_successful {
+            Ok(()) => Ok(Covenant(output)),
+            Err(error) => Err(error),
+        }
+    }
+
     /// Checks a transaction, returning whether or not the transaction is valid.
     ///
     /// The caller must also pass in the [CoinID] and [CoinDataHeight] corresponding to the coin that's being spent, as well as the [Header] of the *previous* block (if this transaction is trying to go into block N, then the header of block N-1). This allows the covenant to access (a committment to) its execution environment, allowing constructs like timelock contracts and colored-coin-like systems.
@@ -101,16 +139,110 @@ impl Covenant {
         }
     }
 
-    /// Checks with respect to a manually instantiated initial heap.
-    pub fn check_raw(&self, args: &[Value]) -> bool {
-        let mut hm = HashMap::new();
-        for (i, v) in args.iter().enumerate() {
-            hm.insert(i as u16, v.clone());
+    /// Runs to the end, with respect to a manually instantiated initial heap.
+    /// This is for testing, when we do not have a transaction.
+    pub fn debug_run_without_transaction(&self, args: &[Value]) -> bool {
+        let mut hashmap: HashMap<u16, Value> = HashMap::new();
+
+        args.iter().enumerate().for_each(|(index, value)| {
+            hashmap.insert(index as u16, value.clone());
+        });
+
+        match self.to_ops() {
+            Ok(ops) => {
+                let mut executor: Executor = Executor::new(ops, hashmap);
+
+                while executor.pc < executor.instrs.len() {
+                    if executor.stack.is_empty() {
+                        dbg!("Stack (step) is empty.");
+                    } else {
+                        dbg!("Stack (step): {:?}", &executor.stack);
+                    }
+
+                    if executor.heap.is_empty() {
+                        dbg!("Heap (step) is empty.");
+                    } else {
+                        dbg!("Heap (step): {:?}", &executor.heap);
+                    }
+
+                    if executor.step().is_none() {
+                        return false;
+                    }
+                }
+
+                if executor.stack.is_empty() {
+                    dbg!("Stack (final) is empty.");
+                } else {
+                    dbg!("Stack (final): {:?}", &executor.stack);
+                }
+
+                if executor.heap.is_empty() {
+                    dbg!("Heap (final) is empty.");
+                } else {
+                    dbg!("Heap (final): {:?}", &executor.heap);
+                }
+
+                executor.stack.pop().map(|f| f.into_bool()).unwrap_or_default()
+            },
+            Err(error) => {
+                dbg!("While converting inputs to OpCodes, we hit a decode error: {}", error);
+
+                false
+            },
         }
-        if let Ok(ops) = self.to_ops() {
-            Executor::new(ops, hm).run_to_end()
-        } else {
-            false
+    }
+
+    /// Runs to the end, with respect to a manually instantiated initial heap.
+    /// This is for testing, when we do not have a transaction.
+    /// This method outputs a tuple containing the stack and the heap.
+    pub fn debug_run_outputting_stack_and_heap(&self, args: &[Value]) -> Option<(Vec<Value>, HashMap<u16, Value>)> {
+        let mut hashmap: HashMap<u16, Value> = HashMap::new();
+
+        args.iter().enumerate().for_each(|(index, value)| {
+            hashmap.insert(index as u16, value.clone());
+        });
+
+        match self.to_ops() {
+            Ok(ops) => {
+                let mut executor: Executor = Executor::new(ops, hashmap);
+
+                while executor.pc < executor.instrs.len() {
+                    if executor.stack.is_empty() {
+                        dbg!("Stack (step) is empty.");
+                    } else {
+                        dbg!("Stack (step): {:?}", &executor.stack);
+                    }
+
+                    if executor.heap.is_empty() {
+                        dbg!("Heap (step) is empty.");
+                    } else {
+                        dbg!("Heap (step): {:?}", &executor.heap);
+                    }
+
+                    if executor.step().is_none() {
+                        return None;
+                    }
+                }
+
+                if executor.stack.is_empty() {
+                    dbg!("Stack (final) is empty.");
+                } else {
+                    dbg!("Stack (final): {:?}", &executor.stack);
+                }
+
+                if executor.heap.is_empty() {
+                    dbg!("Heap (final) is empty.");
+                } else {
+                    dbg!("Heap (final): {:?}", &executor.heap);
+                }
+
+                Some((executor.stack, executor.heap))
+            },
+            Err(error) => {
+                dbg!("While converting inputs to OpCodes, we hit a decode error: {}", error);
+
+                None
+            },
         }
     }
 
@@ -131,7 +263,7 @@ impl Covenant {
             OpCode::LoadImm(1),
             OpCode::SigEOk(32),
         ])
-        .unwrap()
+        .expect("Could not create a legacy ed25519 signature checking covenant.")
     }
 
     /// Returns a new ed25519 signature checking covenant, which checks the *nth* signature when spent as the nth input.
@@ -146,35 +278,16 @@ impl Covenant {
             OpCode::LoadImm(1),
             OpCode::SigEOk(32),
         ])
-        .unwrap()
-    }
-
-    /// Create a Covenant from a slice of OpCodes.
-    pub fn from_ops(ops: &[OpCode]) -> Result<Self, EncodeError> {
-        let mut output: Vec<u8> = Vec::new();
-        // go through output
-        for op in ops {
-            output.extend_from_slice(&op.encode()?)
-        }
-        Ok(Covenant(output))
+            .expect("Could not create a new ed25519 signature checking covenant.")
     }
 
     pub fn always_true() -> Self {
         Covenant::from_ops(&[OpCode::PushI(1u32.into())]).unwrap()
     }
 
-    /// Converts to a vector of OpCodes.
-    pub fn to_ops(&self) -> Result<Vec<OpCode>, DecodeError> {
-        let mut collected = Vec::with_capacity(128);
-        let mut rdr = self.0.as_slice();
-        while !rdr.is_empty() {
-            collected.push(OpCode::decode(&mut rdr)?);
-        }
-        Ok(collected)
-    }
-
     pub fn weight(&self) -> Result<u128, DecodeError> {
         let ops = self.to_ops()?;
+
         Ok(opcodes_weight(&ops))
     }
 }
@@ -298,6 +411,7 @@ impl Executor {
                 return false;
             }
         }
+
         self.stack.pop().map(|f| f.into_bool()).unwrap_or_default()
     }
 
@@ -327,34 +441,65 @@ impl Executor {
             self.pc += 1;
             // eprintln!("running {:?}", op);
             match op {
-                OpCode::Noop => {}
+                OpCode::Noop => {
+                    dbg!("NoOp");
+                }
                 // arithmetic
                 OpCode::Add => self.do_binop(|x, y| {
+                    dbg!("Addition, First: {}", &x);
+                    dbg!("Addition, Second: {}", &y);
+
                     Some(Value::Int(x.into_int()?.overflowing_add(y.into_int()?).0))
                 })?,
                 OpCode::Sub => self.do_binop(|x, y| {
+                    dbg!("Subtraction, First: {}", &x);
+                    dbg!("Subtraction, Second: {}", &y);
+
                     Some(Value::Int(x.into_int()?.overflowing_sub(y.into_int()?).0))
                 })?,
                 OpCode::Mul => self.do_binop(|x, y| {
+                    dbg!("Multiplication, First: {}", &x);
+                    dbg!("Multiplication, Second: {}", &y);
+
                     Some(Value::Int(x.into_int()?.overflowing_mul(y.into_int()?).0))
                 })?,
                 OpCode::Div => self
-                    .do_binop(|x, y| Some(Value::Int(x.into_int()?.checked_div(y.into_int()?)?)))?,
+                    .do_binop(|x, y| {
+                        dbg!("Division, First: {}", &x);
+                        dbg!("Division, Second: {}", &y);
+
+                        Some(Value::Int(x.into_int()?.checked_div(y.into_int()?)?))
+                    })?,
                 OpCode::Rem => self
-                    .do_binop(|x, y| Some(Value::Int(x.into_int()?.checked_rem(y.into_int()?)?)))?,
+                    .do_binop(|x, y| {
+                        dbg!("Remainder, First: {}", &x);
+                        dbg!("Remainder, Second: {}", &y);
+
+                        Some(Value::Int(x.into_int()?.checked_rem(y.into_int()?)?))
+                    })?,
                 // logic
                 OpCode::And => {
-                    self.do_binop(|x, y| Some(Value::Int(x.into_int()? & y.into_int()?)))?
+                    self.do_binop(|x, y| {
+                        Some(Value::Int(x.into_int()? & y.into_int()?))
+                    })?
                 }
                 OpCode::Or => {
-                    self.do_binop(|x, y| Some(Value::Int(x.into_int()? | y.into_int()?)))?
+                    self.do_binop(|x, y| {
+                        Some(Value::Int(x.into_int()? | y.into_int()?))
+                    })?
                 }
                 OpCode::Xor => {
-                    self.do_binop(|x, y| Some(Value::Int(x.into_int()? ^ y.into_int()?)))?
+                    self.do_binop(|x, y| {
+                        Some(Value::Int(x.into_int()? ^ y.into_int()?))
+                    })?
                 }
-                OpCode::Not => self.do_monop(|x| Some(Value::Int(!x.into_int()?)))?,
+                OpCode::Not => self.do_monop(|x| {
+                    Some(Value::Int(!x.into_int()?))
+                })?,
                 OpCode::Eql => self.do_binop(|x, y| match (x, y) {
                     (Value::Int(x), Value::Int(y)) => {
+                        dbg!("Equality, First: {}", &x);
+                        dbg!("Equality, Second: {}", &y);
                         if x == y {
                             Some(Value::Int(1u32.into()))
                         } else {
@@ -364,6 +509,9 @@ impl Executor {
                     _ => None,
                 })?,
                 OpCode::Lt => self.do_binop(|x, y| {
+                    dbg!("Less than, First: {}", &x);
+                    dbg!("Less than, Second: {}", &y);
+
                     let x = x.into_int()?;
                     let y = y.into_int()?;
                     if x < y {
@@ -373,6 +521,9 @@ impl Executor {
                     }
                 })?,
                 OpCode::Gt => self.do_binop(|x, y| {
+                    dbg!("Greater than, First: {}", &x);
+                    dbg!("Greater than, Second: {}", &y);
+
                     let x = x.into_int()?;
                     let y = y.into_int()?;
                     if x > y {
@@ -384,178 +535,312 @@ impl Executor {
                 OpCode::Shl => self.do_binop(|x, offset| {
                     let x = x.into_int()?;
                     let offset = offset.into_int()?;
+
                     Some(Value::Int(x.wrapping_shl(offset.as_u32())))
                 })?,
                 OpCode::Shr => self.do_binop(|x, offset| {
                     let x = x.into_int()?;
                     let offset = offset.into_int()?;
+
                     Some(Value::Int(x.wrapping_shr(offset.as_u32())))
                 })?,
                 // cryptography
                 OpCode::Hash(n) => self.do_monop(|to_hash| {
-                    let to_hash = to_hash.into_bytes()?;
-                    if to_hash.len() > n as usize {
+                    let bytes: CatVec<u8, 256> = to_hash.into_bytes()?;
+
+                    if bytes.len() > n as usize {
                         return None;
                     }
-                    let to_hash: Vec<u8> = to_hash.into();
-                    let hash = tmelcrypt::hash_single(&to_hash);
+
+                    let byte_vector: Vec<u8> = bytes.into();
+                    let hash: tmelcrypt::HashVal = tmelcrypt::hash_single(&byte_vector);
+
+                    dbg!("Hash: {}", &hash.0);
+
                     Some(Value::from_bytes(&hash.0))
                 })?,
                 OpCode::SigEOk(n) => self.do_triop(|message, public_key, signature| {
                     //println!("SIGEOK({:?}, {:?}, {:?})", message, public_key, signature);
-                    let pk = public_key.into_bytes()?;
-                    if pk.len() > 32 {
+                    let public_key_bytes: CatVec<u8, 256> = public_key.into_bytes()?;
+
+                    if public_key_bytes.len() > 32 {
                         return Some(Value::from_bool(false));
                     }
-                    let pk_b: Vec<u8> = pk.into();
-                    let public_key = tmelcrypt::Ed25519PK::from_bytes(&pk_b)?;
-                    let message = message.into_bytes()?;
-                    if message.len() > n as usize {
+
+                    let public_key_byte_vector: Vec<u8> = public_key_bytes.into();
+                    let public_key: tmelcrypt::Ed25519PK = tmelcrypt::Ed25519PK::from_bytes(&public_key_byte_vector)?;
+                    let message_bytes: CatVec<u8, 256> = message.into_bytes()?;
+
+                    if message_bytes.len() > n as usize {
                         return None;
                     }
-                    let message: Vec<u8> = message.into();
-                    let signature = signature.into_bytes()?;
-                    if signature.len() > 64 {
+
+                    let message_byte_vector: Vec<u8> = message_bytes.into();
+                    let signature_bytes: CatVec<u8, 256> = signature.into_bytes()?;
+
+                    if signature_bytes.len() > 64 {
                         return Some(Value::from_bool(false));
                     }
-                    let signature: Vec<u8> = signature.into();
-                    Some(Value::from_bool(public_key.verify(&message, &signature)))
+
+                    let signature_byte_vector: Vec<u8> = signature_bytes.into();
+
+                    Some(Value::from_bool(public_key.verify(&message_byte_vector, &signature_byte_vector)))
                 })?,
                 // storage access
                 OpCode::Store => {
-                    let addr = self.stack.pop()?.into_u16()?;
-                    let val = self.stack.pop()?;
-                    self.heap.insert(addr, val);
+                    let address: u16 = self.stack.pop()?.into_u16()?;
+                    let value: Value = self.stack.pop()?;
+
+                    dbg!("Storing {} at address: {} on the heap.", &value, &address);
+
+                    self.heap.insert(address, value);
                 }
                 OpCode::Load => {
-                    let addr = self.stack.pop()?.into_u16()?;
-                    let res = self.heap.get(&addr)?.clone();
+                    let address: u16 = self.stack.pop()?.into_u16()?;
+                    let res: Value = self.heap.get(&address)?.clone();
+
+                    dbg!("Loading {} from address: {} from the heap.", &res, &address);
+
                     self.stack.push(res)
                 }
                 OpCode::StoreImm(idx) => {
-                    let val = self.stack.pop()?;
-                    self.heap.insert(idx, val);
+                    let value: Value = self.stack.pop()?;
+
+                    dbg!("Storing {} at index {} immutably on the heap.", &value, &idx);
+
+                    self.heap.insert(idx, value);
                 }
                 OpCode::LoadImm(idx) => {
                     let res = self.heap.get(&idx)?.clone();
+
+                    dbg!("Loading {} from index {} immutably from the heap.", &res, &idx);
+
                     self.stack.push(res)
                 }
                 // vector operations
                 OpCode::VRef => self.do_binop(|vec, idx| {
-                    let idx = idx.into_u16()? as usize;
+                    let idx: usize = idx.into_u16()? as usize;
+
+                    dbg!("Loading index {} from VM vector containing {} onto the stack.", &idx, &vec);
+
                     Some(vec.into_vector()?.get(idx)?.clone())
                 })?,
                 OpCode::VSet => self.do_triop(|vec, idx, value| {
-                    let idx = idx.into_u16()? as usize;
-                    let mut vec = vec.into_vector()?;
+                    let idx: usize = idx.into_u16()? as usize;
+                    let mut vec: CatVec<Value, 32> = vec.into_vector()?;
+
+                    dbg!("Overwriting index {} of a VM vector containing {} with {}", &idx, &vec, &value);
+
                     *vec.get_mut(idx)? = value;
+
                     Some(Value::Vector(vec))
                 })?,
                 OpCode::VAppend => self.do_binop(|v1, v2| {
                     let mut v1 = v1.into_vector()?;
                     let v2 = v2.into_vector()?;
+
+                    dbg!("Appending a vector that contains {} to a vector that contains {}", &v2, &v1);
+
                     v1.append(v2);
+
                     Some(Value::Vector(v1))
                 })?,
-                OpCode::VSlice => self.do_triop(|vec, i, j| {
-                    let i = i.into_u16()? as usize;
-                    let j = j.into_u16()? as usize;
+                OpCode::VSlice => self.do_triop(|vec, beginning_value, end_value| {
+                    let beginning: usize = beginning_value.into_u16()? as usize;
+                    let end: usize = end_value.into_u16()? as usize;
+
                     match vec {
                         Value::Vector(vec) => {
-                            if j >= vec.len() || j <= i {
+                            let is_end_greater_or_equal_to_vector_length: bool = end >= vec.len();
+                            let is_end_less_than_or_equal_to_beginning: bool = end <= beginning;
+
+                            if is_end_greater_or_equal_to_vector_length || is_end_less_than_or_equal_to_beginning {
+                                dbg!("Tried to create a VM slice with invalid bounds. Returning an empty VM vector.");
+
                                 Some(Value::Vector(Default::default()))
                             } else {
-                                Some(Value::Vector(vec.tap_mut(|vec| vec.slice_into(i..j))))
+                                dbg!("Returning a slice from {} to {} from the VM vector containing: {}", beginning, end, &vec);
+
+                                Some(Value::Vector(vec.tap_mut(|vec| vec.slice_into(beginning..end))))
                             }
                         }
-                        _ => None,
+                        _ => {
+                            dbg!("Tried to call VSlice on something that was not a VM vector (Value::Vector).");
+
+                            None
+                        },
                     }
                 })?,
                 OpCode::VLength => self.do_monop(|vec| match vec {
-                    Value::Vector(vec) => Some(Value::Int(U256::from(vec.len() as u64))),
-                    _ => None,
+                    Value::Vector(vec) => {
+                        let length: usize = vec.len();
+
+                        dbg!("VM vector is of length: {}", length);
+
+                        Some(Value::Int(U256::from(length as u64)))
+                    },
+                    _ => {
+                        dbg!("Tried to call VLength on something that was not a VM vector (Value::Vector).");
+
+                        None
+                    },
                 })?,
-                OpCode::VEmpty => self.stack.push(Value::Vector(Default::default())),
+                OpCode::VEmpty => {
+                    dbg!("Creating a new empty vector on the stack.");
+
+                    self.stack.push(Value::Vector(Default::default()))
+                },
                 OpCode::VPush => self.do_binop(|vec, item| {
-                    let mut vec = vec.into_vector()?;
+                    let mut vec: CatVec<Value, 32> = vec.into_vector()?;
+
+                    dbg!("Pushing: {} into a VM vector that contains: {}.", &item, &vec);
+
                     vec.push_back(item);
+
                     Some(Value::Vector(vec))
                 })?,
                 OpCode::VCons => self.do_binop(|item, vec| {
-                    let mut vec = vec.into_vector()?;
+                    let mut vec: CatVec<Value, 32> = vec.into_vector()?;
+
+                    dbg!("Inserting: {} at index 0 of a VM vector that contains: {}", &item, &vec);
+
                     vec.insert(0, item);
+
                     Some(Value::Vector(vec))
                 })?,
                 // bit stuff
-                OpCode::BEmpty => self.stack.push(Value::Bytes(Default::default())),
+                OpCode::BEmpty => {
+                    dbg!("Creating a new empty byte vector on the stack.");
+
+                    self.stack.push(Value::Bytes(Default::default()))
+                },
                 OpCode::BPush => self.do_binop(|vec, val| {
-                    let mut vec = vec.into_bytes()?;
-                    let val = val.into_int()?;
+                    let mut vec: CatVec<u8, 256> = vec.into_bytes()?;
+                    let val: U256 = val.into_int()?;
+
+                    dbg!("Pushing: {} into a byte vector containing: {}", &val, &vec);
+
                     vec.push_back(*val.low() as u8);
+
                     Some(Value::Bytes(vec))
                 })?,
                 OpCode::BCons => self.do_binop(|item, vec| {
-                    let mut vec = vec.into_bytes()?;
+                    let mut vec: CatVec<u8, 256> = vec.into_bytes()?;
+
+                    dbg!("Inserting: {} at index 0 of a byte vector that contains: {}", &item, &vec);
+
                     vec.insert(0, item.into_truncated_u8()?);
+
                     Some(Value::Bytes(vec))
                 })?,
                 OpCode::BRef => self.do_binop(|vec, idx| {
-                    let idx = idx.into_u16()? as usize;
+                    let idx: usize = idx.into_u16()? as usize;
+
+                    dbg!("Loading index {} from a stack byte vector containing {} onto the stack.", &idx, &vec);
+
                     Some(Value::Int(vec.into_bytes()?.get(idx).copied()?.into()))
                 })?,
                 OpCode::BSet => self.do_triop(|vec, idx, value| {
-                    let idx = idx.into_u16()? as usize;
-                    let mut vec = vec.into_bytes()?;
+                    let idx: usize = idx.into_u16()? as usize;
+                    let mut vec: CatVec<u8, 256> = vec.into_bytes()?;
+
+                    dbg!("Overwriting index {} of a byte vector containing {} with {}", &idx, &vec, &value);
+
                     *vec.get_mut(idx)? = value.into_truncated_u8()?;
+
                     Some(Value::Bytes(vec))
                 })?,
                 OpCode::BAppend => self.do_binop(|v1, v2| {
-                    let mut v1 = v1.into_bytes()?;
-                    let v2 = v2.into_bytes()?;
-                    // eprintln!("append {} to {}", v1.len(), v2.len());
+                    let mut v1: CatVec<u8, 256> = v1.into_bytes()?;
+                    let v2: CatVec<u8, 256> = v2.into_bytes()?;
+
+                    dbg!("Appending a vector that contains {} to a vector that contains {}", &v2, &v1);
+
                     v1.append(v2);
+
                     Some(Value::Bytes(v1))
                 })?,
-                OpCode::BSlice => self.do_triop(|vec, i, j| {
-                    let i = i.into_u16()? as usize;
-                    let j = j.into_u16()? as usize;
+                OpCode::BSlice => self.do_triop(|vec, beginning_value, end_value| {
+                    let beginning: usize = beginning_value.into_u16()? as usize;
+                    let end: usize = end_value.into_u16()? as usize;
+
                     match vec {
                         Value::Bytes(mut vec) => {
-                            if j >= vec.len() || j <= i {
+                            let is_end_greater_or_equal_to_vector_length: bool = end >= vec.len();
+                            let is_end_less_than_or_equal_to_beginning: bool = end <= beginning;
+
+                            if is_end_greater_or_equal_to_vector_length || is_end_less_than_or_equal_to_beginning {
+                                dbg!("Tried to create a byte slice with invalid bounds. Returning an empty byte vector.");
+
                                 Some(Value::Bytes(Default::default()))
                             } else {
-                                vec.slice_into(i..j);
+                                dbg!("Returning a byte slice from {} to {} from the byte vector containing: {}", beginning, end, &vec);
+
+                                vec.slice_into(beginning..end);
+
                                 Some(Value::Bytes(vec))
                             }
                         }
-                        _ => None,
+                        _ => {
+                            dbg!("Tried to call VSlice on something that was not a VM vector (Value::Vector).");
+
+                            None
+                        },
                     }
                 })?,
                 OpCode::BLength => self.do_monop(|vec| match vec {
-                    Value::Bytes(vec) => Some(Value::Int(U256::from(vec.len() as u64))),
-                    _ => None,
+                    Value::Bytes(vec) => {
+                        let length: usize = vec.len();
+
+                        dbg!("Byte vector is of length: {}", length);
+
+                        Some(Value::Int(U256::from(length as u64)))
+                    },
+                    _ => {
+                        dbg!("Tried to call BLength on something that was not a byte vector (Value::Bytes).");
+
+                        None
+                    },
                 })?,
                 // control flow
                 OpCode::Bez(jgap) => {
                     let top = self.stack.pop()?;
+
                     if top.into_int() == Some(0u32.into()) {
+                        dbg!("In a call to Bez, the top of the stack was zero. Skipping to {}", &jgap);
+
                         self.pc += jgap as usize;
+
                         return Some(());
+                    } else {
+                        dbg!("In a call to Bez, the top of the stack was not zero. It was {}. Not skipping any operations.", &jgap);
                     }
                 }
                 OpCode::Bnz(jgap) => {
                     let top = self.stack.pop()?;
+
                     if top.into_int() != Some(0u32.into()) {
+                        dbg!("In a call to Bnz, the top of the stack was not zero. Skipping to {}", &jgap);
+
                         self.pc += jgap as usize;
                         return Some(());
+                    } else {
+                        dbg!("In a call to Bnz, the top of the stack was not zero. It was {}. Not skipping any operations.", &jgap);
                     }
                 }
                 OpCode::Jmp(jgap) => {
+                    dbg!("Jumping ahead to instruction number {}", &jgap);
+
                     self.pc += jgap as usize;
                     return Some(());
                 }
                 OpCode::Loop(iterations, op_count) => {
-                    if iterations > 0 && op_count > 0 {
+                    let is_iterations_positive: bool = iterations > 0;
+                    let is_op_count_positive: bool = op_count > 0;
+
+                    if is_iterations_positive && is_op_count_positive {
+                        dbg!("In a call to Loop, iterations and op_count were positive. Looping {} times.", iterations);
+
                         self.loop_state.push(LoopState {
                             // start after loop instruction
                             begin: self.pc,
@@ -565,43 +850,114 @@ impl Executor {
                             iterations_left: iterations - 1,
                         });
                     } else {
+                        if !is_iterations_positive {
+                            dbg!("In a call to Loop, iterations was not positive: {}. Skipping loop.", iterations);
+                        } else if !is_op_count_positive {
+                            dbg!("In a call to Loop, op_count was not positive: {}. Skipping loop.", iterations);
+                        } else {
+                            dbg!("In a call to Loop, neither iterations: {}, nor op_count were positive: {}. Skipping loop.", iterations, op_count);
+                        }
+
                         return None;
                     }
                 }
                 // Conversions
-                OpCode::BtoI => self.do_monop(|x| {
-                    let bytes = x.into_bytes()?;
-                    let bytes: Vec<u8> = bytes.into();
+                OpCode::BtoI => self.do_monop(|input_byte_vector| {
+                    dbg!("Converting bytes {} into an integer.", &input_byte_vector);
 
-                    Some(Value::Int(U256::from_be_bytes(bytes.try_into().ok()?)))
+                    let bytes = input_byte_vector.into_bytes()?;
+                    let bytes_vector: Vec<u8> = bytes.into();
+
+                    let byte_vector_option: Option<[u8; 32]> = bytes_vector.try_into().ok();
+
+                    match byte_vector_option {
+                        Some(byte_vector) => {
+                            dbg!("In a call to BtoI, successfully converted input bytes to an integer.");
+
+                            Some(Value::Int(U256::from_be_bytes(byte_vector)))
+                        },
+                        None => {
+                            dbg!("In a call to BtoI, failed to convert input bytes to an integer.");
+
+                            None
+                        },
+                    }
                 })?,
-                OpCode::ItoB => self.do_monop(|x| {
-                    let n = x.into_int()?;
-                    Some(Value::Bytes(n.to_be_bytes().into()))
+                OpCode::ItoB => self.do_monop(|input_integer| {
+                    let number_option: Option<U256> = input_integer.into_int();
+
+                    // I may not need this match, because it may not be able to fail, given that only positive integers are available.
+
+                    match number_option {
+                        Some(number) => {
+                            dbg!("In a call to ItoB, successfully converted input integer to bytes.");
+
+                            Some(Value::Bytes(number.to_be_bytes().into()))
+                        },
+                        None => {
+                            dbg!("In a call to ItoB, failed to convert input integer to bytes.");
+
+                            None
+                        },
+                    }
                 })?,
                 // literals
                 OpCode::PushB(bts) => {
-                    let bts = Value::from_bytes(&bts);
-                    self.stack.push(bts);
+                    let bytes: Value = Value::from_bytes(&bts);
+
+                    dbg!("Pushing a byte vector containing {} onto the stack.", &bytes);
+
+                    self.stack.push(bytes);
                 }
-                OpCode::PushI(num) => self.stack.push(Value::Int(num)),
-                OpCode::PushIC(num) => self.stack.push(Value::Int(num)),
-                OpCode::TypeQ => self.do_monop(|x| match x {
-                    Value::Int(_) => Some(Value::Int(0u32.into())),
-                    Value::Bytes(_) => Some(Value::Int(1u32.into())),
-                    Value::Vector(_) => Some(Value::Int(2u32.into())),
+                OpCode::PushI(num) => {
+
+                    let number: Value = Value::Int(num);
+
+                    dbg!("Pushing the integer {} onto the stack.", &number);
+
+                    self.stack.push(number)
+                },
+                OpCode::PushIC(number) => {
+                    let integer: Value = Value::Int(number);
+
+                    dbg!("PushIC called. Pushing the integer {} onto the stack.", &integer);
+
+                    self.stack.push(integer)
+                },
+                OpCode::TypeQ => self.do_monop(|input| {
+                    match input {
+                        Value::Int(integer) => {
+                            dbg!("In a call to TypeQ, the input was an integer: {}. Returning 0 to the stack.", integer);
+
+                            Some(Value::Int(0u32.into()))
+                        }
+                        Value::Bytes(byte_vector) => {
+                            dbg!("In a call to TypeQ, the input was a byte vector containing: {}. Returning 1 to the stack.", byte_vector);
+
+                            Some(Value::Int(1u32.into()))
+                        }
+                        Value::Vector(vector) => {
+                            dbg!("In a call to TypeQ, the input was a vector containing: {}. Returning 2 to the stack.", vector);
+
+                            Some(Value::Int(2u32.into()))
+                        }
+                    }
                 })?,
                 // dup
                 OpCode::Dup => {
-                    let val = self.stack.pop()?;
-                    self.stack.push(val.clone());
-                    self.stack.push(val);
+                    let value: Value = self.stack.pop()?;
+
+                    dbg!("Dup called. Duplicating: {} on the stack.", &value);
+
+                    self.stack.push(value.clone());
+                    self.stack.push(value);
                 }
             }
             Some(())
         };
-        let res = inner();
+        let res: Option<()> = inner();
         self.update_pc_state();
+
         res
     }
 }
@@ -810,9 +1166,13 @@ mod tests {
     #[test]
     fn stack_overflow() {
         let mut data = Vec::new();
-        for _ in 0..100000 {
+
+        let range = 0..100000;
+
+        range.into_iter().for_each(|_index| {
             data.push(0xb0)
-        }
+        });
+
         dontcrash(&data.to_vec())
     }
     #[test]
@@ -920,4 +1280,4 @@ mod tests {
             }
         }
     }
-}
+}
\ No newline at end of file
